@@ -2,7 +2,9 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2024-2025 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2025 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -57,7 +59,7 @@ public:
     }
 
     void SetContent(const std::string& key,
-                    const std::string& value)
+                    const Json::Value& value)
     {
       that_.SetContent(key, value);
     }
@@ -166,8 +168,16 @@ private:
     }
   }
 
+  // void SetContent(const std::string& key,
+  //                 const std::string& value)
+  // {
+  //   boost::mutex::scoped_lock lock(mutex_);
+  //   content_[key] = value;
+  //   UpdateContent(content_);
+  // }
+
   void SetContent(const std::string& key,
-                  const std::string& value)
+                  const Json::Value& value)
   {
     boost::mutex::scoped_lock lock(mutex_);
     content_[key] = value;
@@ -491,8 +501,22 @@ static void CheckStowAnswer(const Json::Value& response,
 }
 
 
+static void AddResourceForJobContent(Json::Value& resourcesForJobContent /* out */, Orthanc::ResourceType resourceType, const std::string& resourceId)
+{
+  const char* resourceGroup = Orthanc::GetResourceTypeText(resourceType, true, true);
+
+  if (!resourcesForJobContent.isMember(resourceGroup))
+  {
+    resourcesForJobContent[resourceGroup] = Json::arrayValue;
+  }
+  
+  resourcesForJobContent[resourceGroup].append(resourceId);
+}
+
+
 static void ParseStowRequest(std::list<std::string>& instances /* out */,
                              std::map<std::string, std::string>& httpHeaders /* out */,
+                             Json::Value& resourcesForJobContent /* out */,
                              const Json::Value& body /* in */)
 {
   static const char* RESOURCES = "Resources";
@@ -528,27 +552,34 @@ static void ParseStowRequest(std::list<std::string>& instances /* out */,
     }
 
     // Test whether this resource is an instance
-    Json::Value tmp;
-    if (OrthancPlugins::RestApiGet(tmp, "/instances/" + resource, false))
+    Json::Value tmpResource;
+    Json::Value tmpInstances;
+    if (OrthancPlugins::RestApiGet(tmpResource, "/instances/" + resource, false))
     {
-      AddInstance(instances, tmp);
+      // AddInstance(instances, tmpResource);
+      instances.push_back(resource);
+      AddResourceForJobContent(resourcesForJobContent, Orthanc::ResourceType_Instance, resource);
     }
     // This was not an instance, successively try with series/studies/patients
-    else if ((OrthancPlugins::RestApiGet(tmp, "/series/" + resource, false) &&
-              OrthancPlugins::RestApiGet(tmp, "/series/" + resource + "/instances", false)) ||
-             (OrthancPlugins::RestApiGet(tmp, "/studies/" + resource, false) &&
-              OrthancPlugins::RestApiGet(tmp, "/studies/" + resource + "/instances", false)) ||
-             (OrthancPlugins::RestApiGet(tmp, "/patients/" + resource, false) &&
-              OrthancPlugins::RestApiGet(tmp, "/patients/" + resource + "/instances", false)))
+    else if ((OrthancPlugins::RestApiGet(tmpResource, "/series/" + resource, false) &&
+              OrthancPlugins::RestApiGet(tmpInstances, "/series/" + resource + "/instances?expand=false", false)) ||
+             (OrthancPlugins::RestApiGet(tmpResource, "/studies/" + resource, false) &&
+              OrthancPlugins::RestApiGet(tmpInstances, "/studies/" + resource + "/instances?expand=false", false)) ||
+             (OrthancPlugins::RestApiGet(tmpResource, "/patients/" + resource, false) &&
+              OrthancPlugins::RestApiGet(tmpInstances, "/patients/" + resource + "/instances?expand=false", false)))
     {
-      if (tmp.type() != Json::arrayValue)
+      if (tmpInstances.type() != Json::arrayValue)
       {
         throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
       }
 
-      for (Json::Value::ArrayIndex j = 0; j < tmp.size(); j++)
+      AddResourceForJobContent(resourcesForJobContent, Orthanc::StringToResourceType(tmpResource["Type"].asString().c_str()), resource);
+
+      for (Json::Value::ArrayIndex j = 0; j < tmpInstances.size(); j++)
       {
-        AddInstance(instances, tmp[j]);
+        // AddInstance(instances, tmpInstances[j]);
+        instances.push_back(tmpInstances[j].asString());
+        AddResourceForJobContent(resourcesForJobContent, Orthanc::ResourceType_Instance, tmpInstances[j].asString());
       }
     }
     else
@@ -571,15 +602,16 @@ private:
     Action_Cancel
   };
   
-  boost::mutex                             mutex_;
-  std::string                              serverName_;
-  std::vector<std::string>                 instances_;
-  OrthancPlugins::HttpClient::HttpHeaders  headers_;
-  std::string                              boundary_;
-  size_t                                   position_;
-  Action                                   action_;
-  size_t                                   networkSize_;
-  bool                                     debug_;
+  boost::mutex                 mutex_;
+  std::string                  serverName_;
+  std::vector<std::string>     instances_;
+  OrthancPlugins::HttpHeaders  headers_;
+  std::string                  boundary_;
+  size_t                       position_;
+  Action                       action_;
+  size_t                       networkSize_;
+  bool                         debug_;
+  Json::Value                  resourcesForJobContent_;
 
   bool ReadNextInstance(std::string& dicom,
                         JobContext& context)
@@ -698,6 +730,8 @@ private:
       {
         boost::mutex::scoped_lock lock(that_.mutex_);
         context.SetContent("InstancesCount", boost::lexical_cast<std::string>(that_.instances_.size()));
+        context.SetContent("Resources", that_.GetResourcesForJobContent());
+        context.SetContent("Server", that_.GetServerName());
         serverName = that_.serverName_;
         
         startPosition = that_.position_;        
@@ -710,7 +744,7 @@ private:
         client->AddHeaders(that_.headers_);
       }
 
-      OrthancPlugins::HttpClient::HttpHeaders answerHeaders;
+      OrthancPlugins::HttpHeaders answerHeaders;
       Json::Value answerBody;
 
       assert(client.get() != NULL);
@@ -730,7 +764,7 @@ private:
            * chunked transfers, or a version of Orthanc <= 1.7.2 that
            * supports chunk transfers, but cannot receive multipart
            * messages larger than 2GB. The latter problem is fixed by:
-           * https://hg.orthanc-server.com/orthanc/rev/36257d6f348f
+           * https://orthanc.uclouvain.be/hg/orthanc/rev/36257d6f348f
            **/
           if (client->IsChunkedTransfersAllowed())
           {
@@ -792,14 +826,16 @@ private:
 public:
   StowClientJob(const std::string& serverName,
                 const std::list<std::string>& instances,
-                const OrthancPlugins::HttpClient::HttpHeaders& headers) :
+                const OrthancPlugins::HttpHeaders& headers,
+                const Json::Value& resourcesForJobContent) :
     SingleFunctionJob("DicomWebStowClient"),
     serverName_(serverName),
     headers_(headers),
     position_(0),
     action_(Action_None),
     networkSize_(0),
-    debug_(false)
+    debug_(false),
+    resourcesForJobContent_(resourcesForJobContent)
   {
     SetFactory(*this);
 
@@ -835,6 +871,16 @@ public:
   {
     debug_ = debug;
   }
+
+  const Json::Value& GetResourcesForJobContent()
+  {
+    return resourcesForJobContent_;
+  }
+
+  const std::string& GetServerName()
+  {
+    return serverName_;
+  }
 };
 
 
@@ -863,12 +909,13 @@ void StowClient(OrthancPluginRestOutput* output,
 
   std::list<std::string> instances;
   std::map<std::string, std::string> httpHeaders;
-  ParseStowRequest(instances, httpHeaders, body);
+  Json::Value resourcesForJobContent;
+  ParseStowRequest(instances, httpHeaders, resourcesForJobContent, body);
 
-  OrthancPlugins::LogInfo("Sending " + boost::lexical_cast<std::string>(instances.size()) +
-                          " instances using STOW-RS to DICOMweb server: " + serverName);
+  LOG(INFO) << "Sending " << boost::lexical_cast<std::string>(instances.size()) <<
+               " instances using STOW-RS to DICOMweb server: " << serverName;
 
-  std::unique_ptr<StowClientJob> job(new StowClientJob(serverName, instances, httpHeaders));
+  std::unique_ptr<StowClientJob> job(new StowClientJob(serverName, instances, httpHeaders, resourcesForJobContent));
 
   bool debug;
   if (OrthancPlugins::LookupBooleanValue(debug, body, "Debug"))
